/*
 * aTunes 1.9.0
 * Copyright (C) 2006-2008 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel.controllers.osd;

import java.awt.Point;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseListener;

import javax.swing.Timer;

import net.sourceforge.atunes.gui.images.ImageLoader;
import net.sourceforge.atunes.gui.views.dialogs.OSDDialog;
import net.sourceforge.atunes.kernel.Kernel;
import net.sourceforge.atunes.kernel.controllers.model.DialogController;
import net.sourceforge.atunes.kernel.modules.podcast.PodcastFeedEntry;
import net.sourceforge.atunes.kernel.modules.radio.Radio;
import net.sourceforge.atunes.kernel.modules.repository.audio.AudioFile;
import net.sourceforge.atunes.kernel.utils.AudioFilePictureUtils;
import net.sourceforge.atunes.model.AudioObject;
import net.sourceforge.atunes.utils.StringUtils;
import net.sourceforge.atunes.utils.TimeUtils;

/**
 * The Class OSDDialogController.
 */
public class OSDDialogController extends DialogController<OSDDialog> {

	/**
	 * The Enum Direction.
	 */
	private enum Direction {

		/** The UP. */
		UP,

		/** The DOWN. */
		DOWN
	}

	/** The Constant STEP. */
	private static final int STEP = 4;

	/** The timer. */
	Timer timer;

	/** The width. */
	private int width = Toolkit.getDefaultToolkit().getScreenSize().width / 3;

	/** The height. */
	private int height = 84;

	/** The location to stay. */
	Point locationToStay = new Point(width, Toolkit.getDefaultToolkit().getScreenSize().height - (int) (height * 1.7));

	/** The hidden location. */
	Point hiddenLocation = new Point(locationToStay.x, Toolkit.getDefaultToolkit().getScreenSize().height + 4);

	/**
	 * Instantiates a new oSD dialog controller.
	 * 
	 * @param dialogControlled
	 *            the dialog controlled
	 */
	public OSDDialogController(OSDDialog dialogControlled) {
		super(dialogControlled);
		addBindings();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sourceforge.atunes.kernel.controllers.model.Controller#addBindings()
	 */
	@Override
	protected void addBindings() {
		MouseListener listener = new OSDDialogMouseListener(getDialogControlled(), this);
		getDialogControlled().addMouseListener(listener);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sourceforge.atunes.kernel.controllers.model.Controller#addStateBindings()
	 */
	@Override
	protected void addStateBindings() {
		// Nothing to do
	}

	/**
	 * Gets the timer.
	 * 
	 * @return the timer
	 */
	private Timer getTimer() {
		return new Timer(10, new ActionListener() {

			private boolean showAnimation = Kernel.getInstance().state.isAnimateOSD();
			private Direction direction = Direction.UP;
			private int show = Kernel.getInstance().state.getOsdDuration() * 100;
			private int wait;
			private int position;

			@Override
			public void actionPerformed(@SuppressWarnings("unused")
			ActionEvent e) {
				if (showAnimation && direction == Direction.UP && getDialogControlled().getLocation().y > locationToStay.y) {
					// Move up
					getDialogControlled().setLocation(locationToStay.x, getDialogControlled().getLocation().y - STEP);
				} else {
					direction = Direction.DOWN;
					// Show
					if (++wait > show) {
						if (showAnimation && direction == Direction.DOWN && getDialogControlled().getLocation().y < hiddenLocation.y) {
							// Workaround for Gnome	
							if (position == getDialogControlled().getLocation().y) {
								showAnimation = false;
							}
							position = getDialogControlled().getLocation().y;
							// Move down
							getDialogControlled().setLocation(locationToStay.x, getDialogControlled().getLocation().y + STEP);
						} else {
							// Finished
							direction = Direction.UP;
							wait = 0;
							getDialogControlled().setVisible(false);
							timer.stop();
						}
					}
				}

			}
		});
	}

	@Override
	protected OSDDialog getDialogControlled() {
		return super.getDialogControlled();
	}

	/**
	 * Hide dialog and stop animation.
	 */
	public void hideDialogAndStopAnimation() {
		if (timer != null && timer.isRunning()) {
			getDialogControlled().setVisible(false);
			timer.stop();
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sourceforge.atunes.kernel.controllers.model.Controller#notifyReload()
	 */
	@Override
	protected void notifyReload() {
		// Nothing to do
	}

	/**
	 * Sets the transparent.
	 * 
	 * @param transparent
	 *            the new transparent
	 */
	public void setTransparent(boolean transparent) {
		getDialogControlled().setVisible(false);

		// Sets the window trannsparency
		getDialogControlled().setTransparent(transparent);

		// Add rounded borders too if transparent
		getDialogControlled().setRoundedBorders(transparent);
	}

	/**
	 * Show osd.
	 * 
	 * @param audioObject
	 *            the audio object
	 */
	public void showOSD(AudioObject audioObject) {

		if (audioObject == null) {
			return;
		}

		// If the OSD is already visible stop animation
		hideDialogAndStopAnimation();

		if (audioObject instanceof PodcastFeedEntry) {
			getDialogControlled().setImage(ImageLoader.RSS);
			getDialogControlled().setLine1(audioObject.getTitle());
			getDialogControlled().setLine2(audioObject.getUrl());
			getDialogControlled().setLine3(audioObject.getArtist());
		} else if (audioObject instanceof Radio) {
			getDialogControlled().setImage(ImageLoader.RADIO);
			getDialogControlled().setLine1(audioObject.getTitle());
			getDialogControlled().setLine2(audioObject.getUrl());
			getDialogControlled().setLine3("");
		} else if (audioObject instanceof AudioFile) {
			getDialogControlled().setImage(AudioFilePictureUtils.getImageForAudioFile((AudioFile) audioObject, -1, -1));
			getDialogControlled().setLine1(StringUtils.getString(audioObject.getTitleOrFileName(), " (", TimeUtils.seconds2String(audioObject.getDuration()), ")"));
			getDialogControlled().setLine2(audioObject.getAlbum());
			getDialogControlled().setLine3(audioObject.getArtist());
		} else {
			throw new IllegalArgumentException(audioObject + " is not a audio file, radio or podcast feed entry");
		}
		getDialogControlled().setLocation(Kernel.getInstance().state.isAnimateOSD() ? hiddenLocation : locationToStay);
		getDialogControlled().setVisible(true);
		// see bug 1864517
		getDialogControlled().repaint();
		timer = getTimer();
		timer.start();
	}
}
