/*
 * aTunes 1.9.0
 * Copyright (C) 2006-2008 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel.handlers;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Random;

import javax.swing.JOptionPane;

import net.sourceforge.atunes.gui.model.PlayListTableModel;
import net.sourceforge.atunes.gui.views.controls.playList.PlayListTable;
import net.sourceforge.atunes.gui.views.dialogs.CopyProgressDialog;
import net.sourceforge.atunes.gui.views.dialogs.FileSelectionDialog;
import net.sourceforge.atunes.kernel.ControllerProxy;
import net.sourceforge.atunes.kernel.Kernel;
import net.sourceforge.atunes.kernel.controllers.navigation.NavigationControllerViews;
import net.sourceforge.atunes.kernel.executors.BackgroundExecutor;
import net.sourceforge.atunes.kernel.executors.processes.ExportFilesProcess;
import net.sourceforge.atunes.kernel.modules.device.DeviceConnectionListener;
import net.sourceforge.atunes.kernel.modules.device.DeviceConnectionMonitor;
import net.sourceforge.atunes.kernel.modules.device.DeviceCopyFinishListener;
import net.sourceforge.atunes.kernel.modules.device.DeviceDisconnectionListener;
import net.sourceforge.atunes.kernel.modules.device.DeviceDisconnectionMonitor;
import net.sourceforge.atunes.kernel.modules.repository.LoaderListener;
import net.sourceforge.atunes.kernel.modules.repository.Repository;
import net.sourceforge.atunes.kernel.modules.repository.RepositoryLoader;
import net.sourceforge.atunes.kernel.modules.repository.audio.AudioFile;
import net.sourceforge.atunes.kernel.modules.repository.model.Artist;
import net.sourceforge.atunes.kernel.modules.repository.model.Folder;
import net.sourceforge.atunes.kernel.modules.repository.model.Genre;
import net.sourceforge.atunes.kernel.modules.state.ApplicationState;
import net.sourceforge.atunes.utils.LanguageTool;
import net.sourceforge.atunes.utils.StringUtils;
import net.sourceforge.atunes.utils.log4j.LogCategories;
import net.sourceforge.atunes.utils.log4j.Logger;

/**
 * The Class DeviceHandler.
 */
public class DeviceHandler implements LoaderListener, DeviceCopyFinishListener, DeviceConnectionListener, DeviceDisconnectionListener {

	/** The instance. */
	private static DeviceHandler instance = new DeviceHandler();

	/** The logger. */
	private Logger logger = new Logger();

	/** The device repository. */
	private Repository deviceRepository;

	/** The temp device repository. */
	private Repository tempDeviceRepository;

	/** The current loader. */
	private RepositoryLoader currentLoader;

	/** The device path. */
	private File devicePath;

	/** The state. */
	private ApplicationState state = Kernel.getInstance().state;

	/**
	 * Instantiates a new device handler.
	 */
	private DeviceHandler() {
		// Nothing to do
	}

	/**
	 * Gets the single instance of DeviceHandler.
	 * 
	 * @return single instance of DeviceHandler
	 */
	public static DeviceHandler getInstance() {
		return instance;
	}

	/**
	 * Fills the device with songs until the specified memory is left.
	 * 
	 * @param leaveFreeLong
	 *            Memory to leave free
	 */
	public void fillWithRandomSongs(long leaveFreeLong) {
		long leaveFree = leaveFreeLong;
		logger.debug(LogCategories.HANDLER, new String[] { Long.toString(leaveFree) });

		// Get reference to Repository songs
		List<AudioFile> songs = RepositoryHandler.getInstance().getSongs();

		// Songs selected
		List<AudioFile> songsSelected = new ArrayList<AudioFile>();

		// Initialize random generator
		Random r = new Random(new Date().getTime());

		// Get free space in device
		long deviceFreeSpace = deviceRepository.getFolders().get(0).getFreeSpace();
		if (leaveFree >= 0)
			leaveFree = (leaveFree) * 1048576; // Transfrom to same unit
		else
			leaveFree = 0; // User typed nonsense value. So fill his device ...

		// Not enough space avaible
		if (leaveFree > deviceFreeSpace) {
			VisualHandler.getInstance().showErrorDialog(LanguageTool.getString("NOT_ENOUGH_SPACE_ON_DEVICE"));
			return;
		}

		// Get n songs
		for (int i = 0; leaveFree < deviceFreeSpace; i++) {
			// Get song number
			int number = r.nextInt(songs.size());

			// Add selectedSong
			songsSelected.add(songs.get(number));

			// Compute free space after song has been added
			deviceFreeSpace = deviceFreeSpace - songs.get(number).length();
		}

		// Write files
		copyFilesToDevice(songsSelected);
	}

	/**
	 * Connect device.
	 */
	public void connectDevice() {
		FileSelectionDialog dialog = VisualHandler.getInstance().getFileSelectionDialog(true);
		dialog.setTitle(LanguageTool.getString("SELECT_DEVICE"));
		dialog.startDialog();
		if (!dialog.isCanceled()) {
			File dir = dialog.getSelectedDir();

			VisualHandler.getInstance().showProgressBar(true, null);
			this.retrieveDevice(dir);
		}
	}

	/**
	 * Copy files to mp3 device.
	 * 
	 * @param files
	 *            Files to be written to a mp3 device
	 */
	@SuppressWarnings("boxing")
	public void copyFilesToDevice(List<AudioFile> files) {
		// Get size of files
		long size = 0;
		for (AudioFile file : files) {
			size = size + file.length();
		}
		// Get free space in device
		long deviceFreeSpace = deviceRepository.getFolders().get(0).getFreeSpace();

		// Get folder structure to use
		int structure;
		if (state.isFlatStructure())
			structure = ExportFilesProcess.FLAT_STRUCTURE;
		else if (state.isArtistStructure())
			structure = ExportFilesProcess.ARTIST_STRUCTURE;
		else
			structure = ExportFilesProcess.FULL_STRUCTURE;

		if (size > deviceFreeSpace) {
			VisualHandler.getInstance().showErrorDialog(LanguageTool.getString("NOT_ENOUGH_SPACE_ON_DEVICE"));
			return;
		}

		CopyProgressDialog exportProgressDialog = VisualHandler.getInstance().getExportProgressDialog();
		exportProgressDialog.getTotalFilesLabel().setText(StringUtils.getString(files.size()));
		exportProgressDialog.getProgressBar().setMaximum(files.size());
		exportProgressDialog.setVisible(true);
		final ExportFilesProcess process = BackgroundExecutor.exportFiles(files, structure, null, deviceRepository.getFolders().get(0).getAbsolutePath(), true);
		process.addFinishListener(this);

		exportProgressDialog.getCancelButton().addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(@SuppressWarnings("unused")
			ActionEvent e) {
				process.notifyCancel();
			}
		});
	}

	/**
	 * Called when a device monitor detects a device connected.
	 * 
	 * @param location
	 *            the location
	 */
	@Override
	public void deviceConnected(String location) {
		if (VisualHandler.getInstance().showConfirmationDialog(LanguageTool.getString("DEVICE_CONNECT_CONFIRMATION"), LanguageTool.getString("DEVICE_DETECTED")) == JOptionPane.OK_OPTION) {
			VisualHandler.getInstance().showProgressBar(true, null);
			this.retrieveDevice(new File(location));
		}
	}

	/**
	 * Called when a device monitor detects a device disconnection.
	 * 
	 * @param location
	 *            the location
	 */
	@Override
	public void deviceDisconnected(String location) {
		VisualHandler.getInstance().showMessage(LanguageTool.getString("DEVICE_DISCONNECTION_DETECTED"));
		disconnectDevice();

		// Start device connection monitor
		DeviceConnectionMonitor.startMonitor();
	}

	/**
	 * Disconnect device.
	 */
	@SuppressWarnings("boxing")
	public void disconnectDevice() {
		VisualHandler.getInstance().hideDeviceInfoOnStatusBar();

		List<Integer> songsToRemove = new ArrayList<Integer>();
		for (int i = 0; i < PlayerHandler.getInstance().getCurrentPlayList().size(); i++) {
			if (PlayerHandler.getInstance().getCurrentPlayList().get(i) instanceof AudioFile) {
				AudioFile song = (AudioFile) PlayerHandler.getInstance().getCurrentPlayList().get(i);
				if (song.getPath().startsWith(this.getDeviceRepository().getFolders().get(0).getPath()))
					songsToRemove.add(i);
			}
		}
		int[] indexes = new int[songsToRemove.size()];
		for (int i = 0; i < songsToRemove.size(); i++)
			indexes[i] = songsToRemove.get(i);

		if (indexes.length > 0) {
			PlayListTable table = VisualHandler.getInstance().getPlayListTable();
			((PlayListTableModel) table.getModel()).removeSongs(indexes);
			VisualHandler.getInstance().getPlayListPanel().getPlayListTable().getSelectionModel().setSelectionInterval(-1, -1);
			PlayListHandler.getInstance().removeSongs(indexes);
		}

		deviceRepository = null;
		notifyFinishRefresh(null);
		logger.info(LogCategories.REPOSITORY, "Device disconnected");
		ControllerProxy.getInstance().getMenuController().setDeviceConnected(false);
	}

	/**
	 * Gets the device artist and album structure.
	 * 
	 * @return the device artist and album structure
	 */
	public Map<String, Artist> getDeviceArtistAndAlbumStructure() {
		if (deviceRepository != null)
			return deviceRepository.getStructure().getTreeStructure();
		return new HashMap<String, Artist>();
	}

	/**
	 * Gets the device's genre structure
	 * 
	 * @return
	 */
	public Map<String, Genre> getDeviceGenreStructure() {
		if (deviceRepository != null)
			return deviceRepository.getStructure().getGenreStructure();
		return new HashMap<String, Genre>();
	}

	/**
	 * Returns a string with total and free space for a dir.
	 * 
	 * @return the device data
	 */
	private String getDeviceData() {
		if (deviceRepository != null) {
			int songs = deviceRepository.countFiles();
			File dir = deviceRepository.getFolders().get(0);
			return StringUtils.getString(Integer.toString(songs), " ", LanguageTool.getString("SONGS"), "  (", LanguageTool.getString("FREE_SPACE"), ": ", StringUtils
					.fromByteToMegaOrGiga(dir.getFreeSpace()), ")");
		}
		return null;
	}

	/**
	 * Gets the device folder structure.
	 * 
	 * @return the device folder structure
	 */
	public Map<String, Folder> getDeviceFolderStructure() {
		if (deviceRepository != null)
			return deviceRepository.getStructure().getFolderStructure();
		return new HashMap<String, Folder>();
	}

	/**
	 * Gets the device repository.
	 * 
	 * @return the device repository
	 */
	public Repository getDeviceRepository() {
		return deviceRepository;
	}

	/**
	 * Gets the device songs.
	 * 
	 * @return the device songs
	 */
	public List<AudioFile> getDeviceSongs() {
		if (deviceRepository != null)
			return deviceRepository.getFilesList();
		return new ArrayList<AudioFile>();
	}

	/**
	 * Checks if is device connected.
	 * 
	 * @return true, if is device connected
	 */
	public boolean isDeviceConnected() {
		return deviceRepository != null;
	}

	/**
	 * Checks if given file is in the device path.
	 * 
	 * @param path
	 *            Absolute path of the file
	 * 
	 * @return true if file is in device, false otherwise
	 */
	public boolean isDevicePath(String path) {
		if (path.contains(devicePath.toString()))
			return true;
		return false;
	}

	/**
	 * Called when finish copy to device.
	 */
	@Override
	public void notifyCopyFinish() {
		// Force device refresh
		refreshDevice();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sourceforge.atunes.kernel.modules.repository.LoaderListener#notifyCurrentPath(java.lang.String)
	 */
	@Override
	public void notifyCurrentPath(@SuppressWarnings("unused")
	String path) {
		// Nothing to do
	}

	/**
	 * Notify device reload.
	 * 
	 * @param loader
	 *            the loader
	 */
	private void notifyDeviceReload(RepositoryLoader loader) {
		if (ControllerProxy.getInstance().getNavigationController() != null) {
			VisualHandler.getInstance().showProgressBar(false, null);
			ControllerProxy.getInstance().getNavigationController().notifyDeviceReload();
			ControllerProxy.getInstance().getMenuController().setDeviceConnected(loader != null);

			// Update status bar info
			if (loader != null)
				VisualHandler.getInstance().showDeviceInfoOnStatusBar(getDeviceData());
			else
				VisualHandler.getInstance().hideDeviceInfoOnStatusBar();

			if (loader != null)
				// Switch to device view in navigator
				ControllerProxy.getInstance().getNavigationController().setNavigationView(NavigationControllerViews.DEVICE_VIEW);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sourceforge.atunes.kernel.modules.repository.LoaderListener#notifyFileLoaded()
	 */
	@Override
	public void notifyFileLoaded() {
		// Nothing to do
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sourceforge.atunes.kernel.modules.repository.LoaderListener#notifyFilesInRepository(int)
	 */
	@Override
	public void notifyFilesInRepository(@SuppressWarnings("unused")
	int files) {
		// Nothing to do
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sourceforge.atunes.kernel.modules.repository.LoaderListener#notifyFinishRead(net.sourceforge.atunes.kernel.modules.repository.RepositoryLoader)
	 */
	@Override
	public void notifyFinishRead(RepositoryLoader loader) {
		deviceRepository = loader.getRepository();
		logger.info(LogCategories.REPOSITORY, "Device read");
		notifyDeviceReload(loader);
		// Start device disconnection monitor
		DeviceDisconnectionMonitor.startMonitor();
		DeviceDisconnectionMonitor.addListener(this);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sourceforge.atunes.kernel.modules.repository.LoaderListener#notifyFinishRefresh(net.sourceforge.atunes.kernel.modules.repository.RepositoryLoader)
	 */
	@Override
	public void notifyFinishRefresh(RepositoryLoader loader) {
		if (loader != null)
			tempDeviceRepository = loader.getRepository();
		deviceRepository = tempDeviceRepository;
		tempDeviceRepository = null;
		notifyDeviceReload(loader);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sourceforge.atunes.kernel.modules.repository.LoaderListener#notifyRemainingTime(long)
	 */
	@Override
	public void notifyRemainingTime(@SuppressWarnings("unused")
	long time) {
		// Nothing to do
	}

	/**
	 * Refresh device.
	 */
	public void refreshDevice() {
		VisualHandler.getInstance().showProgressBar(true, null);
		logger.info(LogCategories.REPOSITORY, "Refreshing device");
		currentLoader = new RepositoryLoader(deviceRepository.getFolders(), true);
		currentLoader.addRepositoryLoaderListener(this);
		currentLoader.setOldRepository(deviceRepository);
		currentLoader.start();
	}

	/**
	 * Retrieve device.
	 * 
	 * @param path
	 *            the path
	 */
	public void retrieveDevice(File path) {
		logger.info(LogCategories.REPOSITORY, StringUtils.getString("Reading device mounted on ", path));
		List<File> folders = new ArrayList<File>();
		folders.add(path);
		currentLoader = new RepositoryLoader(folders, false);
		currentLoader.addRepositoryLoaderListener(this);
		setDevicePath(path);
		currentLoader.start();
	}

	/**
	 * Set the device path.
	 * 
	 * @param path
	 *            Device path (absolute)
	 */
	private void setDevicePath(File path) {
		devicePath = path;
	}

}
