/*
 * aTunes 1.9.0
 * Copyright (C) 2006-2008 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel.handlers;

import java.util.Arrays;

import net.sourceforge.atunes.kernel.ControllerProxy;
import net.sourceforge.atunes.kernel.controllers.playList.PlayListController;
import net.sourceforge.atunes.kernel.modules.mplayer.MPlayerHandler;
import net.sourceforge.atunes.kernel.modules.playlist.PlayList;
import net.sourceforge.atunes.utils.LanguageTool;
import net.sourceforge.atunes.utils.StringUtils;
import net.sourceforge.atunes.utils.log4j.LogCategories;
import net.sourceforge.atunes.utils.log4j.Logger;

/**
 * The Class PlayerHandler.
 */
public abstract class PlayerHandler {

	/** The instance. */
	private static PlayerHandler instance = new MPlayerHandler();

	/** The Constant PREVIOUS. */
	protected static final int PREVIOUS = 1;

	/** The Constant PLAY. */
	protected static final int PLAY = 2;

	/** The Constant STOP. */
	protected static final int STOP = 3;

	/** The Constant NEXT. */
	protected static final int NEXT = 4;

	/** The Constant SEEK. */
	protected static final int SEEK = 5;

	/** The last button pressed. */
	protected int lastButtonPressed = -1;

	/** The muted. */
	protected boolean muted;

	/** The logger. */
	protected Logger logger = new Logger();

	/** The current play list. */
	protected PlayList currentPlayList;

	/** The current duration. */
	long currentDuration;

	/** The volume. */
	protected int volume = 50; // Per cent

	/** The balance. */
	protected float balance; // -1, +1

	/** The use normalisation. */
	protected boolean useNormalisation;

	/** The shuffle. */
	protected boolean shuffle;

	/** The repeat. */
	protected boolean repeat;

	/** The karaoke. */
	protected boolean karaoke;

	// Mplayer equalizer settings
	/** The equalizer. */
	protected float[] equalizer;

	/** The paused. */
	protected boolean paused;

	/** The play list controller. */
	protected PlayListController playListController;

	/**
	 * Instantiates a new player handler.
	 */
	protected PlayerHandler() {
		logger.debug(LogCategories.PLAYER);

		this.playListController = ControllerProxy.getInstance().getPlayListController();
		currentPlayList = new PlayList();
		VisualHandler.getInstance().updateStatusBar(LanguageTool.getString("STOPPED"));
	}

	/**
	 * Gets the single instance of PlayerHandler.
	 * 
	 * @return single instance of PlayerHandler
	 */
	public static PlayerHandler getInstance() {
		return instance;
	}

	/**
	 * Finish.
	 */
	public abstract void finish();

	/**
	 * Gets the current duration.
	 * 
	 * @return the current duration
	 */
	public long getCurrentDuration() {
		return currentDuration;
	}

	/**
	 * Gets the current play list.
	 * 
	 * @return the current play list
	 */
	public final PlayList getCurrentPlayList() {
		return currentPlayList;
	}

	/**
	 * Gets the equalizer.
	 * 
	 * @return the equalizer
	 */
	public final float[] getEqualizer() {
		if (equalizer == null) {
			return null;
		}
		return Arrays.copyOf(equalizer, equalizer.length);
	}

	/**
	 * Gets the volume.
	 * 
	 * @return the volume
	 */
	public final int getVolume() {
		return volume;
	}

	/**
	 * Checks if is karaoke.
	 * 
	 * @return true, if is karaoke
	 */
	public boolean isKaraoke() {
		return karaoke;
	}

	/**
	 * Checks if is mute.
	 * 
	 * @return true, if is mute
	 */
	public final boolean isMute() {
		return muted;
	}

	/**
	 * Checks if is playing.
	 * 
	 * @return true, if is playing
	 */
	public abstract boolean isPlaying();

	/**
	 * Checks if is repeat.
	 * 
	 * @return the repeat
	 */
	public boolean isRepeat() {
		return repeat;
	}

	/**
	 * Checks if is shuffle.
	 * 
	 * @return the shuffle
	 */
	public boolean isShuffle() {
		return shuffle;
	}

	/**
	 * Checks if is use normalisation.
	 * 
	 * @return true, if is use normalisation
	 */
	public boolean isUseNormalisation() {
		return useNormalisation;
	}

	/**
	 * Next.
	 * 
	 * @param autoNext
	 *            the auto next
	 */
	public abstract void next(boolean autoNext);

	/**
	 * Notify player error.
	 * 
	 * @param e
	 *            the e
	 */
	public final void notifyPlayerError(Exception e) {
		logger.error(LogCategories.PLAYER, StringUtils.getString("Player Error: ", e));
		e.printStackTrace(System.out);
		VisualHandler.getInstance().showErrorDialog(e.getMessage());
	}

	/**
	 * Play.
	 * 
	 * @param buttonPressed
	 *            the button pressed
	 */
	public abstract void play(boolean buttonPressed);

	/**
	 * Previous.
	 */
	public abstract void previous();

	/**
	 * Seek.
	 * 
	 * @param position
	 *            the position
	 */
	public abstract void seek(double position);

	/**
	 * Sets the balance.
	 * 
	 * @param value
	 *            the new balance
	 */
	public abstract void setBalance(float value);

	/**
	 * Sets the current duration.
	 * 
	 * @param currentDuration
	 *            the new current duration
	 */
	public void setCurrentDuration(long currentDuration) {
		this.currentDuration = currentDuration;
	}

	/**
	 * Sets the current play list.
	 * 
	 * @param currentPlayList
	 *            the new current play list
	 */
	public final void setCurrentPlayList(PlayList currentPlayList) {
		this.currentPlayList = currentPlayList;
	}

	/**
	 * Sets the duration.
	 * 
	 * @param time
	 *            the new duration
	 */
	public final void setDuration(long time) {
		logger.debug(LogCategories.PLAYER, new String[] { Long.toString(time) });

		ControllerProxy.getInstance().getPlayerControlsController().setMaxDuration(time);

		VisualHandler.getInstance().getFullScreenFrame().setMaxDuration(time);
	}

	/**
	 * Sets the equalizer.
	 * 
	 * @param equalizer
	 *            the new equalizer
	 */
	public void setEqualizer(float[] equalizer) {
		if (equalizer == null) {
			this.equalizer = null;
		} else {
			this.equalizer = Arrays.copyOf(equalizer, equalizer.length);
		}
	}

	/**
	 * Sets the karaoke.
	 * 
	 * @param karaoke
	 *            the new karaoke
	 */
	public void setKaraoke(boolean karaoke) {
		this.karaoke = karaoke;
	}

	/**
	 * Sets the mute.
	 * 
	 * @param mute
	 *            the new mute
	 */
	public abstract void setMute(boolean mute);

	/**
	 * Sets the play list position to play.
	 * 
	 * @param pos
	 *            the new play list position to play
	 */
	public final void setPlayListPositionToPlay(int pos) {
		currentPlayList.setCurrentAudioObject(pos);
	}

	/**
	 * Sets the repeat.
	 * 
	 * @param enable
	 *            the new repeat
	 */
	@SuppressWarnings("boxing")
	public final void setRepeat(boolean enable) {
		repeat = enable;
		logger.info(LogCategories.PLAYER, StringUtils.getString("Repeat enabled: ", enable));
	}

	/**
	 * Sets the shuffle.
	 * 
	 * @param enable
	 *            the new shuffle
	 */
	@SuppressWarnings("boxing")
	public final void setShuffle(boolean enable) {
		shuffle = enable;
		logger.info(LogCategories.PLAYER, StringUtils.getString("Shuffle enabled: ", enable));
	}

	/**
	 * Sets the time.
	 * 
	 * @param time
	 *            the new time
	 */
	public void setTime(long time) {
		ControllerProxy.getInstance().getPlayerControlsController().setTime(time, currentDuration);
		VisualHandler.getInstance().getFullScreenFrame().setTime(time, currentDuration);
	}

	/**
	 * Sets the use normalisation.
	 * 
	 * @param useNormalisation
	 *            the new use normalisation
	 */
	public void setUseNormalisation(boolean useNormalisation) {
		this.useNormalisation = useNormalisation;
	}

	/**
	 * Sets the volume.
	 * 
	 * @param perCent
	 *            the new volume
	 */
	public abstract void setVolume(int perCent);

	/**
	 * Stop.
	 * 
	 * @param userStopped
	 *            the user stopped
	 */
	public abstract void stop(boolean userStopped);

	/**
	 * Volume down.
	 */
	public final void volumeDown() {
		logger.debug(LogCategories.PLAYER);

		setVolume(getVolume() - 5);
		VisualHandler.getInstance().setVolume(getVolume());
	}

	/**
	 * Volume up.
	 */
	public final void volumeUp() {
		logger.debug(LogCategories.PLAYER);

		setVolume(getVolume() + 5);
		VisualHandler.getInstance().setVolume(getVolume());
	}

	/**
	 * Checks if is paused.
	 * 
	 * @return true, if is paused
	 */
	public boolean isPaused() {
		return paused;
	}

}
