/*
 * aTunes 1.9.0
 * Copyright (C) 2006-2008 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel.modules.audioscrobbler;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Locale;

import javax.swing.ImageIcon;

import net.sourceforge.atunes.kernel.utils.XMLUtils;
import net.sourceforge.atunes.utils.StringUtils;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * The Class AudioScrobblerAlbum.
 */
public class AudioScrobblerAlbum {

	/** The Constant df. */
	private static final ThreadLocal<SimpleDateFormat> df = new ThreadLocal<SimpleDateFormat>() {
		@Override
		protected SimpleDateFormat initialValue() {
			return new SimpleDateFormat("d MMM yyyy, HH:mm", Locale.US);
		}
	};

	/** The artist. */
	private String artist;

	/** The title. */
	private String title;

	/** The url. */
	private String url;

	/** The release date string. */
	private String releaseDateString;

	/** The big cover url. */
	private String bigCoverURL;

	/** The cover url. */
	private String coverURL;

	/** The small cover url. */
	private String smallCoverURL;

	/** The tracks. */
	private List<AudioScrobblerTrack> tracks;

	// Used by renderers
	/** The cover. */
	private ImageIcon cover;

	/**
	 * Gets the album.
	 * 
	 * @param xml
	 *            the xml
	 * 
	 * @return the album
	 */
	protected static AudioScrobblerAlbum getAlbum(Document xml) {
		AudioScrobblerAlbum album = new AudioScrobblerAlbum();

		Element element = (Element) xml.getElementsByTagName("album").item(0);

		album.artist = element.getAttribute("artist");
		album.title = element.getAttribute("title");
		album.url = XMLUtils.getChildElementContent(element, "url");
		album.releaseDateString = XMLUtils.getChildElementContent(element, "releasedate").trim();
		album.bigCoverURL = XMLUtils.getChildElementContent(XMLUtils.getChildElement(element, "coverart"), "large");
		album.coverURL = XMLUtils.getChildElementContent(XMLUtils.getChildElement(element, "coverart"), "medium");
		album.smallCoverURL = XMLUtils.getChildElementContent(XMLUtils.getChildElement(element, "coverart"), "small");
		NodeList tracks = ((Element) element.getElementsByTagName("tracks").item(0)).getElementsByTagName("track");

		album.tracks = new ArrayList<AudioScrobblerTrack>();
		for (int i = 0; i < tracks.getLength(); i++)
			album.tracks.add(AudioScrobblerTrack.getTrack((Element) tracks.item(i)));

		return album;
	}

	/**
	 * Gets the album list.
	 * 
	 * @param xml
	 *            the xml
	 * 
	 * @return the album list
	 */
	protected static AudioScrobblerAlbumList getAlbumList(Document xml) {
		List<AudioScrobblerAlbum> albums = new ArrayList<AudioScrobblerAlbum>();

		Element element = (Element) xml.getElementsByTagName("topalbums").item(0);
		String artist = element.getAttribute("artist");
		NodeList list = element.getElementsByTagName("album");

		for (int i = 0; i < list.getLength(); i++) {
			Element alb = (Element) list.item(i);
			AudioScrobblerAlbum album = new AudioScrobblerAlbum();
			album.artist = artist;
			album.title = XMLUtils.getChildElementContent(alb, "name");
			album.url = XMLUtils.getChildElementContent(alb, "url");
			album.releaseDateString = XMLUtils.getChildElementContent(alb, "releasedate").trim();
			album.bigCoverURL = XMLUtils.getChildElementContent(XMLUtils.getChildElement(alb, "image"), "large");
			album.coverURL = XMLUtils.getChildElementContent(XMLUtils.getChildElement(alb, "image"), "medium");
			album.smallCoverURL = XMLUtils.getChildElementContent(XMLUtils.getChildElement(alb, "image"), "small");
			albums.add(album);
		}

		AudioScrobblerAlbumList albumList = new AudioScrobblerAlbumList();
		albumList.setArtist(artist);
		albumList.setAlbums(albums);
		return albumList;
	}

	/**
	 * Gets the artist.
	 * 
	 * @return the artist
	 */
	public String getArtist() {
		return artist;
	}

	/**
	 * Gets the artist url.
	 * 
	 * @return the artist url
	 */
	public String getArtistUrl() {
		return url.substring(0, url.lastIndexOf('/'));
	}

	/**
	 * Gets the big cover url.
	 * 
	 * @return the bigCoverURL
	 */
	public String getBigCoverURL() {
		return bigCoverURL;
	}

	/**
	 * Gets the cover.
	 * 
	 * @return the cover
	 */
	public ImageIcon getCover() {
		return cover;
	}

	/**
	 * Gets the cover url.
	 * 
	 * @return the cover url
	 */
	public String getCoverURL() {
		return coverURL;
	}

	/**
	 * Gets the release date.
	 * 
	 * @return the release date
	 */
	public Date getReleaseDate() {
		try {
			return df.get().parse(releaseDateString);
		} catch (ParseException e) {
			return null;
		}
	}

	/**
	 * Gets the release date string.
	 * 
	 * @return the releaseDateString
	 */
	public String getReleaseDateString() {
		return releaseDateString;
	}

	/**
	 * Gets the small cover url.
	 * 
	 * @return the small cover url
	 */
	public String getSmallCoverURL() {
		return smallCoverURL;
	}

	/**
	 * Gets the title.
	 * 
	 * @return the title
	 */
	public String getTitle() {
		return title;
	}

	/**
	 * Gets the tracks.
	 * 
	 * @return the tracks
	 */
	public List<AudioScrobblerTrack> getTracks() {
		return tracks;
	}

	/**
	 * Gets the url.
	 * 
	 * @return the url
	 */
	public String getUrl() {
		return url;
	}

	/**
	 * Gets the year.
	 * 
	 * @return the year
	 */
	public String getYear() {
		Date releaseDate = getReleaseDate();
		if (releaseDate == null)
			return "";
		Calendar c = Calendar.getInstance();
		c.setTime(releaseDate);
		return Integer.toString(c.get(Calendar.YEAR));
	}

	/**
	 * Sets the artist.
	 * 
	 * @param artist
	 *            the artist to set
	 */
	public void setArtist(String artist) {
		this.artist = artist;
	}

	/**
	 * Sets the big cover url.
	 * 
	 * @param bigCoverURL
	 *            the bigCoverURL to set
	 */
	public void setBigCoverURL(String bigCoverURL) {
		this.bigCoverURL = bigCoverURL;
	}

	/**
	 * Sets the cover.
	 * 
	 * @param cover
	 *            the cover to set
	 */
	public void setCover(ImageIcon cover) {
		this.cover = cover;
	}

	/**
	 * Sets the cover url.
	 * 
	 * @param coverURL
	 *            the coverURL to set
	 */
	public void setCoverURL(String coverURL) {
		this.coverURL = coverURL;
	}

	/**
	 * Sets the release date string.
	 * 
	 * @param releaseDateString
	 *            the releaseDateString to set
	 */
	public void setReleaseDateString(String releaseDateString) {
		this.releaseDateString = releaseDateString;
	}

	/**
	 * Sets the small cover url.
	 * 
	 * @param smallCoverURL
	 *            the smallCoverURL to set
	 */
	public void setSmallCoverURL(String smallCoverURL) {
		this.smallCoverURL = smallCoverURL;
	}

	/**
	 * Sets the title.
	 * 
	 * @param title
	 *            the title to set
	 */
	public void setTitle(String title) {
		this.title = title;
	}

	/**
	 * Sets the tracks.
	 * 
	 * @param tracks
	 *            the tracks to set
	 */
	public void setTracks(List<AudioScrobblerTrack> tracks) {
		this.tracks = tracks;
	}

	/**
	 * Sets the url.
	 * 
	 * @param url
	 *            the url to set
	 */
	public void setUrl(String url) {
		this.url = url;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return StringUtils.getString(artist, " - ", title);
	}
}
