/*
 * aTunes 1.9.0
 * Copyright (C) 2006-2008 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel.modules.mplayer;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;

import net.sourceforge.atunes.gui.views.controls.playList.PlayListTable.PlayState;
import net.sourceforge.atunes.kernel.ControllerProxy;
import net.sourceforge.atunes.kernel.Kernel;
import net.sourceforge.atunes.kernel.handlers.PlayListHandler;
import net.sourceforge.atunes.kernel.handlers.PlayerHandler;
import net.sourceforge.atunes.kernel.handlers.PodcastFeedHandler;
import net.sourceforge.atunes.kernel.handlers.VisualHandler;
import net.sourceforge.atunes.kernel.modules.podcast.PodcastFeedEntry;
import net.sourceforge.atunes.kernel.modules.radio.Radio;
import net.sourceforge.atunes.kernel.modules.repository.audio.AudioFile;
import net.sourceforge.atunes.kernel.utils.ClosingUtils;
import net.sourceforge.atunes.kernel.utils.NativeFunctionsUtils;
import net.sourceforge.atunes.kernel.utils.SystemProperties;
import net.sourceforge.atunes.kernel.utils.TempFolder;
import net.sourceforge.atunes.kernel.utils.SystemProperties.OperatingSystem;
import net.sourceforge.atunes.model.AudioObject;
import net.sourceforge.atunes.utils.LanguageTool;
import net.sourceforge.atunes.utils.StringUtils;
import net.sourceforge.atunes.utils.log4j.LogCategories;

/**
 * The Class MPlayerHandler.
 */
public class MPlayerHandler extends PlayerHandler {

	/**
	 * Command to be executed on Linux systems to launch mplayer. Mplayer should
	 * be in $PATH
	 */
	private static final String LINUX_COMMAND = "mplayer";

	/**
	 * Command to be executed on Windows systems to launch mplayer. Mplayer is
	 * in "win_tools" dir, inside aTunes package
	 */
	private static final String WIN_COMMAND = "win_tools/mplayer.exe";

	/** Command to be executed on Mac systems to launch mplayer. */
	private static final String MACOS_COMMAND = "mplayer";

	/**
	 * Command to be executed on Solaris systems to launch mplayer. Note the
	 * workaround with the options - Java6 on Solaris Express appears to require
	 * these options added separately.
	 */
	private static final String SOLARIS_COMMAND = "mplayer";

	/** The Constant SOLARISOPTAO. */
	private static final String SOLARISOPTAO = "-ao";

	/** The Constant SOLARISOPTTYPE. */
	private static final String SOLARISOPTTYPE = "sun";

	// Arguments for mplayer

	/** Argument to not display more information than needed. */
	private static final String QUIET = "-quiet";

	/** Argument to control mplayer through commands. */
	private static final String SLAVE = "-slave";

	/** Argument to pass mplayer a play list. */
	private static final String PLAYLIST = "-playlist";

	/** Arguments to filter audio output. */
	private static final String AUDIO_FILTER = "-af";

	/** The Constant VOLUME_NORM. */
	private static final String VOLUME_NORM = "volnorm";

	/** The Constant KARAOKE. */
	private static final String KARAOKE = "karaoke";

	/** The Constant EQUALIZER. */
	private static final String EQUALIZER = "equalizer=";

	/** Cache arguments. */
	private static final String CACHE = "-cache";

	/** The Constant CACHE_SIZE. */
	private static final String CACHE_SIZE = "500";

	/** The Constant CACHE_MIN. */
	private static final String CACHE_MIN = "-cache-min";

	/** The Constant CACHE_FILL_SIZE_IN_PERCENT. */
	private static final String CACHE_FILL_SIZE_IN_PERCENT = "7.0";

	/** Prefer IPv4. */
	private static final String PREFER_IPV4 = "-prefer-ipv4";

	/** The process. */
	private Process process;

	/** The command writer. */
	private MPlayerCommandWriter commandWriter;

	/** The m player output reader. */
	private MPlayerOutputReader mPlayerOutputReader;

	/** The m player error reader. */
	private MPlayerErrorReader mPlayerErrorReader;

	/**
	 * Last file cached. When caching a new file, file pointed by this object is
	 * removed from temp folder
	 */
	private File lastFileCached;

	/**
	 * Instantiates a new m player handler.
	 */
	public MPlayerHandler() {
		super();
		if (!testMPlayerAvailability()) {
			notifyPlayerError(new Exception(LanguageTool.getString("MPLAYER_NOT_FOUND")));
		} else {
			commandWriter = new MPlayerCommandWriter(null);
			MPlayerPositionThread positionThread = new MPlayerPositionThread(this);
			positionThread.start();
		}
	}

	/**
	 * Tests if mplayer is present.
	 * 
	 * @return true mplayer is present, false mplayer is missing
	 */
	private static boolean testMPlayerAvailability() {
		if (SystemProperties.OS != OperatingSystem.WINDOWS) {
			BufferedReader stdInput = null;
			try {
				Process p = new ProcessBuilder(LINUX_COMMAND).start();
				stdInput = new BufferedReader(new InputStreamReader(p.getInputStream()));

				while (stdInput.readLine() != null) {
					// Nothing to do
				}

				int code = p.waitFor();
				if (code != 0) {
					return false;
				}
			} catch (Exception e) {
				return false;
			} finally {
				ClosingUtils.close(stdInput);
			}
		}
		return true;
	}

	/**
	 * Stops the player.
	 */
	@Override
	public void finish() {
		stop(false);
		logger.info(LogCategories.PLAYER, "Stopping player");
	}

	/**
	 * Returns a mplayer process to play an audiofile.
	 * 
	 * @param audioObject
	 *            audio object which should be played
	 * 
	 * @return mplayer process
	 * 
	 * @throws IOException
	 *             Signals that an I/O exception has occurred.
	 */
	private Process getProcess(AudioObject audioObject) throws IOException {

		ProcessBuilder pb = new ProcessBuilder();
		List<String> command = new ArrayList<String>();

		boolean isRemoteAudio = !(audioObject instanceof AudioFile || (audioObject instanceof PodcastFeedEntry && Kernel.getInstance().state.isUseDownloadedPodcastFeedEntries() && ((PodcastFeedEntry) audioObject)
				.isDownloaded()));

		command.add(getProcessNameForOS());
		if (SystemProperties.OS == OperatingSystem.SOLARIS) {
			command.add(SOLARISOPTAO);
			command.add(SOLARISOPTTYPE);
		}
		command.add(QUIET);
		command.add(SLAVE);

		// PREFER_IPV4 for radios and podcast entries
		if (isRemoteAudio) {
			command.add(PREFER_IPV4);
		}

		// If a radio has a playlist url add playlist command
		if (audioObject instanceof Radio && ((Radio) audioObject).hasPlaylistUrl()) {
			command.add(PLAYLIST);
		}

		// url
		boolean shortPathName = Kernel.getInstance().state.isUseShortPathNames() && SystemProperties.OS == OperatingSystem.WINDOWS && audioObject instanceof AudioFile;
		String url;
		if (audioObject instanceof PodcastFeedEntry && !isRemoteAudio) {
			url = PodcastFeedHandler.getInstance().getDownloadPath((PodcastFeedEntry) audioObject);
			if (Kernel.getInstance().state.isUseShortPathNames() && SystemProperties.OS == OperatingSystem.WINDOWS) {
				shortPathName = true;
			}
		} else {
			url = audioObject.getUrl();
		}
		if (shortPathName) {
			String shortPath = NativeFunctionsUtils.getShortPathNameW(url);
			command.add(shortPath != null && !shortPath.isEmpty() ? shortPath : url);
		} else {
			command.add(url);
		}

		// Cache for radios and podcast entries
		if (isRemoteAudio) {
			command.add(CACHE);
			command.add(CACHE_SIZE);
			command.add(CACHE_MIN);
			command.add(CACHE_FILL_SIZE_IN_PERCENT);
		}

		if ((audioObject instanceof AudioFile && getEqualizer() != null) || isUseNormalisation() || isKaraoke())
			command.add(AUDIO_FILTER);

		// normalization
		if (isUseNormalisation()) {
			command.add(VOLUME_NORM);
		}

		// Build equalizer command. Mplayer uses 10 bands
		if (audioObject instanceof AudioFile && getEqualizer() != null && !isKaraoke())
			command.add(EQUALIZER + getEqualizer()[0] + ":" + getEqualizer()[1] + ":" + getEqualizer()[2] + ":" + getEqualizer()[3] + ":" + getEqualizer()[4] + ":"
					+ getEqualizer()[5] + ":" + getEqualizer()[6] + ":" + getEqualizer()[7] + ":" + getEqualizer()[8] + ":" + getEqualizer()[9]);

		// karaoke
		if (isKaraoke()) {
			command.add(KARAOKE);
		}

		logger.debug(LogCategories.PLAYER, command.toArray(new String[command.size()]));
		return pb.command(command).start();
	}

	/**
	 * Returns string command to call mplayer.
	 * 
	 * @return the process name for os
	 */
	private String getProcessNameForOS() {
		if (SystemProperties.OS == OperatingSystem.WINDOWS)
			return WIN_COMMAND;
		else if (SystemProperties.OS == OperatingSystem.LINUX)
			return LINUX_COMMAND;
		else if (SystemProperties.OS == OperatingSystem.SOLARIS)
			return SOLARIS_COMMAND;
		else
			return MACOS_COMMAND;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sourceforge.atunes.kernel.handlers.PlayerHandler#isPlaying()
	 */
	@Override
	public boolean isPlaying() {
		return process != null && !paused;
	}

	/**
	 * Goes to next audio file in playlist.
	 * 
	 * @param autoNext
	 *            the auto next
	 */
	@Override
	public void next(boolean autoNext) {
		boolean wasStopped = false;
		AudioObject nextAudioObject = currentPlayList.getNextAudioObjectToPlay();
		if (nextAudioObject != null) {
			try {
				PlayListHandler.getInstance().selectedAudioObjectChanged(nextAudioObject);
				if (process != null || autoNext) {
					stop(false);
					if (!paused) {
						play(nextAudioObject);
						VisualHandler.getInstance().setPlaying(true);
						if (muted)
							setMute(true);
						else
							setVolume(volume);
						setBalance(balance);
						logger.info(LogCategories.PLAYER, StringUtils.getString("Started play of file ", nextAudioObject));
					} else {
						setTime(0);
						wasStopped = true;
					}
				} else {
					wasStopped = true;
					setTime(0);
				}
				if (wasStopped)
					playListController.setSelectedSong(currentPlayList.getCurrentAudioObjectIndex());
				else {
					updateStatusBar(nextAudioObject, false);
					VisualHandler.getInstance().updateTitleBar(nextAudioObject);
					VisualHandler.getInstance().updatePlayState(PlayState.PLAYING);
					logger.debug(LogCategories.PLAYER, Integer.toString(currentPlayList.getCurrentAudioObjectIndex()));
					playListController.setSelectedSong(currentPlayList.getCurrentAudioObjectIndex());
				}
			} catch (Exception e) {
				notifyPlayerError(e);
				stop(false);
			}
		} else { // End of play list. Go to first song and stop
			stop(false);
			currentPlayList.setCurrentAudioObject(0);
			playListController.setSelectedSong(currentPlayList.getCurrentAudioObjectIndex());
			PlayListHandler.getInstance().selectedAudioObjectChanged(currentPlayList.getCurrentAudioObject());
		}

	}

	/**
	 * Notifys the handler that the radio or podcast feed entry has started
	 * playing (MPlayer bug workaround).
	 */
	void notifyRadioOrPodcastFeedEntryStarted() {
		logger.debug(LogCategories.PLAYER, "radio or podcast feed entry has started playing");
		// send volume command
		setVolume(volume);
		// if muted set mute again
		if (muted) {
			setMute(muted);
		}
		logger.debug(LogCategories.PLAYER, "MPlayer bug (ignoring muting and volume settings after streamed file starts playing) workaround applied");
	}

	/**
	 * Starts play process.
	 * 
	 * @param audioObject
	 *            audio object which should be played
	 */
	private void play(AudioObject audioObject) {
		AudioObject audioObjectToPlay = audioObject;
		try {
			// Send stop command in order to try to avoid two mplayer
			// instaces are running at the same time
			commandWriter.sendStopCommand();

			// If cacheFilesBeforePlaying is true and audio object is an audio file, copy it to temp folder
			// and start mplayer process from this copied file
			if (audioObjectToPlay instanceof AudioFile && Kernel.getInstance().state.isCacheFilesBeforePlaying()) {
				// Remove previous cached file
				if (lastFileCached != null)
					TempFolder.removeFile(lastFileCached);

				File tempFile = TempFolder.copyToTempFolder((File) audioObjectToPlay);
				if (tempFile != null) {
					audioObjectToPlay = new AudioFile(tempFile.getAbsolutePath());
					lastFileCached = (File) audioObjectToPlay;
				} else {
					lastFileCached = null;
				}
			}

			// Start the play process
			process = getProcess(audioObjectToPlay);
			commandWriter = new MPlayerCommandWriter(process);
			mPlayerOutputReader = MPlayerOutputReader.newInstance(this, process, audioObjectToPlay);
			mPlayerOutputReader.start();
			mPlayerErrorReader = new MPlayerErrorReader(this, process, audioObjectToPlay);
			mPlayerErrorReader.start();
			commandWriter.sendGetDurationCommand();

			setVolume(volume);
			setBalance(balance);

			// Show OSD
			if (Kernel.getInstance().state.isShowOSD())
				VisualHandler.getInstance().showOSDDialog();

			// If we are playing a podcast, mark entry as listened
			if (audioObjectToPlay instanceof PodcastFeedEntry) {
				((PodcastFeedEntry) audioObjectToPlay).setListened(true);
				// Update pod cast navigator
				ControllerProxy.getInstance().getNavigationController().refreshPodcastFeedTreeContent();
			}

		} catch (Exception e) {
			stop(false);
			notifyPlayerError(e);
		}
	}

	/**
	 * Play music. Pass true if user pressed a button.
	 * 
	 * @param buttonPressed
	 *            the button pressed
	 */
	@Override
	public void play(boolean buttonPressed) {
		if (isPlaying() && buttonPressed) { // Pause
			paused = true;
			VisualHandler.getInstance().setPlaying(false);
			try {
				commandWriter.sendPauseCommand();
				VisualHandler.getInstance().updateStatusBar(LanguageTool.getString("PAUSED"));
				VisualHandler.getInstance().updateTitleBar("");
				VisualHandler.getInstance().updatePlayState(PlayState.PAUSED);
				logger.info(LogCategories.PLAYER, "Pause");
			} catch (Exception e) {
				notifyPlayerError(e);
				stop(false);
			}
		} else {
			// If list is empty, is not playing
			VisualHandler.getInstance().setPlaying(currentPlayList.size() > 0);

			AudioObject nextAudioObject = null;
			try {
				if (paused && buttonPressed) {
					paused = false;
					commandWriter.sendResumeCommand();
					/*
					 * Mplayer volume problem workaround If player was paused,
					 * set volume again as it could be changed when paused
					 */
					commandWriter.sendVolumeCommand(volume);
					/*
					 * End Mplayer volume problem workaround
					 */
					if (!currentPlayList.isEmpty()) {
						updateStatusBar(currentPlayList.get(currentPlayList.getCurrentAudioObjectIndex()), true);
						VisualHandler.getInstance().updateTitleBar(currentPlayList.get(currentPlayList.getCurrentAudioObjectIndex()));
						VisualHandler.getInstance().updatePlayState(PlayState.PAUSED);
						logger.info(LogCategories.PLAYER, "Resumed paused song");
					}
				} else {
					nextAudioObject = currentPlayList.getCurrentAudioObject();
					if (nextAudioObject != null) {
						if (isPlaying() || paused)
							stop(false);

						PlayListHandler.getInstance().selectedAudioObjectChanged(nextAudioObject);
						play(nextAudioObject);
						// Setting volume and balance
						if (muted)
							setMute(true);
						else
							setVolume(volume);
						setBalance(balance);

						updateStatusBar(nextAudioObject, false);
						VisualHandler.getInstance().updateTitleBar(nextAudioObject);
						playListController.setSelectedSong(currentPlayList.getCurrentAudioObjectIndex());
						VisualHandler.getInstance().setPlaying(true);
						logger.info(LogCategories.PLAYER, StringUtils.getString("Started play of file ", nextAudioObject));
					}
				}
				VisualHandler.getInstance().updatePlayState(PlayState.PLAYING);
			} catch (Exception e) {
				notifyPlayerError(e);
				stop(false);
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sourceforge.atunes.kernel.handlers.PlayerHandler#previous()
	 */
	@Override
	/**
	 * Go to previous audio file in playlist.
	 */
	public void previous() {
		boolean wasStopped = false;
		AudioObject previousAudioObject = currentPlayList.getPreviousAudioObjectToPlay();
		if (previousAudioObject != null) {
			try {
				PlayListHandler.getInstance().selectedAudioObjectChanged(previousAudioObject);
				if (process != null) {
					stop(false);
					// Not paused and counter has correct value
					if (!paused) {
						play(previousAudioObject);
						VisualHandler.getInstance().setPlaying(true);
						if (muted)
							setMute(true);
						else
							setVolume(volume);
						setBalance(balance);
						logger.info(LogCategories.PLAYER, StringUtils.getString("Started play of file ", previousAudioObject));
					} else {
						setTime(0);
						wasStopped = true;
					}
				} else {
					wasStopped = true;
					setTime(0);
				}
				playListController.setSelectedSong(currentPlayList.getCurrentAudioObjectIndex());
				if (wasStopped)
					playListController.setSelectedSong(currentPlayList.getCurrentAudioObjectIndex());
				else {
					updateStatusBar(previousAudioObject, false);
					VisualHandler.getInstance().updateTitleBar(previousAudioObject);
					VisualHandler.getInstance().updatePlayState(PlayState.PLAYING);
					playListController.setSelectedSong(currentPlayList.getCurrentAudioObjectIndex());
				}
			} catch (Exception e) {
				notifyPlayerError(e);
				stop(false);
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sourceforge.atunes.kernel.handlers.PlayerHandler#seek(double)
	 */
	@Override
	public void seek(double position) {
		if (!paused) {
			commandWriter.sendSeekCommand(position);
		}
		if (paused) {
			paused = false;
			if (!currentPlayList.isEmpty()) {
				updateStatusBar(currentPlayList.get(currentPlayList.getCurrentAudioObjectIndex()), false);
				VisualHandler.getInstance().updateTitleBar(currentPlayList.get(currentPlayList.getCurrentAudioObjectIndex()));
				VisualHandler.getInstance().updatePlayState(PlayState.PLAYING);
				VisualHandler.getInstance().setPlaying(true);
				logger.info(LogCategories.PLAYER, "Resumed paused song");
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sourceforge.atunes.kernel.handlers.PlayerHandler#setBalance(float)
	 */
	@Override
	public void setBalance(@SuppressWarnings("unused")
	float value) {
		// Nothing to do
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sourceforge.atunes.kernel.handlers.PlayerHandler#setMute(boolean)
	 */
	@Override
	public void setMute(boolean mute) {
		logger.debug(LogCategories.PLAYER, Boolean.toString(mute));

		commandWriter.sendMuteCommand();

		// MPlayer bug: paused, demute, muted -> starts playing
		if (paused && !mute && muted) {
			commandWriter.sendPauseCommand();
			logger.debug(LogCategories.PLAYER, "MPlayer bug (paused, demute, muted -> starts playing) workaround applied");
		}

		muted = mute;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sourceforge.atunes.kernel.handlers.PlayerHandler#setVolume(int)
	 */
	@Override
	public void setVolume(int perCent) {
		if (perCent < 0)
			volume = 0;
		else if (perCent > 100)
			volume = 100;
		else
			volume = perCent;

		// MPlayer bug: paused, volume change -> starts playing
		// If is paused, volume will be sent to mplayer when user resumes playback
		if (!paused) {
			commandWriter.sendVolumeCommand(volume);
		}
	}

	/**
	 * Fades away track before stopping.
	 */
	private void fadeAway() {
		FadeAwayRunnable fadeAwayRunnable = new FadeAwayRunnable(process, getVolume());
		new Thread(fadeAwayRunnable).start();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sourceforge.atunes.kernel.handlers.PlayerHandler#stop(boolean)
	 */
	@Override
	/**
	 * Stops the player and resets values.
	 */
	public void stop(boolean userStopped) {

		boolean activateFadeAway = userStopped && Kernel.getInstance().state.isUseFadeAway();
		if (activateFadeAway && !paused) {
			mPlayerErrorReader.interrupt();
			mPlayerOutputReader.interrupt();
			fadeAway();
		}

		VisualHandler.getInstance().setPlaying(false);
		if (paused)
			paused = false;
		try {
			if (!activateFadeAway) {
				commandWriter.sendStopCommand();
			}
			process = null;
			mPlayerErrorReader = null;
			mPlayerOutputReader = null;
			commandWriter.setProcess(null);
			setTime(0);

			VisualHandler.getInstance().updateStatusBar(LanguageTool.getString("STOPPED"));
			VisualHandler.getInstance().updateTitleBar("");
			VisualHandler.getInstance().updatePlayState(PlayState.STOPPED);
			logger.info(LogCategories.PLAYER, "Stop");
		} catch (Exception e) {
			notifyPlayerError(e);
		}
	}

	/**
	 * Update status bar.
	 * 
	 * @param file
	 *            the file
	 * @param resumed
	 *            the resumed
	 */
	private void updateStatusBar(AudioObject file, boolean resumed) {
		if (resumed || !(file instanceof Radio || file instanceof PodcastFeedEntry))
			VisualHandler.getInstance().updateStatusBar(file);
		else
			VisualHandler.getInstance().updateStatusBar(StringUtils.getString("<html>", LanguageTool.getString("BUFFERING"), " <b>", file.getTitle(), "</b>...</html>"));

	}

	/**
	 * Gets the command writer.
	 * 
	 * @return the command writer
	 */
	public MPlayerCommandWriter getCommandWriter() {
		return commandWriter;
	}

}
